#!/usr/bin/env node
var WebSocketServer = require('websocket').server;
var http = require('http');
var clone = require('clone');
var escape = require('escape-html');

var port = 8037;

var width = 100;
var height = 50;


var server = http.createServer(function(request, response){
	console.log((new Date()) + ' Received request for ' + request.url);
	response.writeHead(404);
	response.end();
});

var grid = initGrid();
var clients = [];
var moves = [];

server.listen(port, function(){
	console.log((new Date()) + ' Server is listening on port ' + port);
	setInterval(tick, 200);
});

wsServer = new WebSocketServer({
	httpServer: server,
	autoAcceptConnections: false
});

function originIsAllowed(origin){
	return true;
}

wsServer.on('request', function(request){
	if(!originIsAllowed(request.origin)){
		request.reject();
		console.log((new Date()) + ' Connection from origin ' + request.origin + ' rejected.');
		return;
	}
	console.log((new Date()) + ' Connection from origin ' + request.origin);

	
	var connection = request.accept(null, request.origin);
	console.log((new Date()) + ' Connection accepted.');
	var playerNum = clients.push({"connection": connection});
	clients[playerNum-1].playerNum = playerNum;
	addPlayerToGrid(playerNum);
	connection.on('message', function(message){
		if(message.type === 'utf8'){
			console.log('Recived Message: ' + message.utf8Data);
			var playerMove=JSON.parse(message.utf8Data);
			if(typeof playerMove.name != 'undefined')
			{
				clients[playerNum-1].name = escape(playerMove.name);
			}
			if(typeof playerMove.move != 'undefined')
			{
				if(['NONE', 'LEFT', 'UP', 'RIGHT', 'DOWN'].indexOf(playerMove.move) !== -1)
				{
					moves[playerNum] = playerMove.move;
				}
			}
		}
		else
		{
			connection.drop();
		}
	});
	function removePlayer(reasonCode, description){
		console.log((new Date()) + ' Peer ' + connection.remoteAddress + ' disconnected.');
		removePlayerFromGrid(playerNum);
	}
	connection.on('close', removePlayer);
});

function initGrid()
{
	var grid =
	{
		"data": [],
		"scores" : []
	}

	for(y=0; y<height; y++)
	{
		grid.data[y] = Array();
		for(x=0; x<width; x++)
		{
			grid.data[y][x] = 0;
		}
	}
	return grid;
}

function addPlayerToGrid(playerNum)
{
	var radius = 7;
	var x = radius + Math.floor(Math.random()*(width-(2*radius)-1));
	var y = radius + Math.floor(Math.random()*(height-(2*radius)-1));
	for(i=x-radius; i<x+radius; i++)
	{
		for(j=y-radius; j<y+radius; j++)
		{
			xDist = i-x;
			yDist = j-y;
			dist = Math.sqrt(xDist*xDist + yDist*yDist);
			if(dist < radius)
			{
				grid.data[j][i] = playerNum;
			}
		}
	}
}

function removePlayerFromGrid(playerNum)
{
	console.log("Removing " + playerNum);
	for(y=0; y<grid.data.length; y++)
	{
		for(x=0; x<grid.data[y].length; x++)
		{
			if(grid.data[y][x] == playerNum)
			{
				grid.data[y][x] = 0;
			}
		}
	}
}

function applyMove(oldGrid, playerNum, playerMove)
{
	var coords = [];
	for(y=0; y<oldGrid.length; y++)
	{
		for(x=0; x<oldGrid[y].length; x++)
		{
			if(oldGrid[y][x] == playerNum)
			{
				coords.push({"x": x, "y": y});
			}
		}
	}
	var newCoords = [];
	coords.forEach(function(coord)
	{
		var newCoord = clone(coord);
		if(playerMove === 'LEFT')
		{
			newCoord.x -= 1;
		}
		else if(playerMove === 'UP')
		{
			newCoord.y -= 1;
		}
		else if(playerMove === 'RIGHT')
		{
			newCoord.x += 1;
		}
		else if(playerMove === 'DOWN')
		{
			newCoord.y += 1;
		}
		newCoords.push(newCoord);
	});
	for(i=0; i<coords.length && i < newCoords.length; i++)
	{
		if(newCoords[i].x < 0 || newCoords[i].x >= width || newCoords[i].y < 0 || newCoords[i].y >= height)
		{
			newCoords[i] = clone(coords[i]);
		}
		if(oldGrid[newCoords[i].y][newCoords[i].x] != playerNum){
			grid.data[newCoords[i].y][newCoords[i].x] = playerNum;
			grid.data[coords[i].y][coords[i].x] = playerNum;
		}
	}
}

function getScores()
{
	var scores = [];
	grid.data.forEach(function(rowData, rowIndex){
		rowData.forEach(function(cellData, cellIndex){
			if(typeof scores[cellData] == 'undefined')
			{
				scores[cellData] = 1;
			}
			else
			{
				scores[cellData]++;
			}
		});
	});
	var result = [];
	scores.forEach(function(value, playerNum){
		var item = {"playerNum": playerNum, "score": value};
		if(typeof clients[playerNum-1] != 'undefined' && typeof clients[playerNum-1].name != 'undefined')
		{
			item.name = clients[playerNum-1].name;
		}
		result.push(item);
	});
	result.sort(function(a,b){
		return a.score-b.score;
	});
	return result;
}

function tick(){
	var oldGrid = clone(grid.data);
	var scores = clone(grid.scores).reverse();
	scores.forEach(function(player, playerIndex){
		if(typeof moves[player.playerNum] != 'undefined')
		{
			applyMove(oldGrid, player.playerNum, moves[player.playerNum]);
		}
	});
	grid.scores = getScores();
	clients.forEach(function(client){
		var message = clone(grid);
		message.playerNum = client.playerNum;
		client.connection.sendUTF(JSON.stringify(message));
	});
	moves = [];
}
